﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Phidget22;
using System.Net.Mail;

namespace TemperatureMonitor
{
    public partial class Form1 : Form
    {
        TemperatureSensor temp;
        int count = 0;
        bool timeOK = true;
        Properties.Settings defaultSettings;
        public Form1()
        {
            InitializeComponent();
        }

        #region FORM_EVENTS
        private void Form1_Load(object sender, EventArgs e)
        {
            defaultSettings = Properties.Settings.Default;
            Log.Enable(LogLevel.Info, "temperatureMonitorLog.log");
            temp = new TemperatureSensor();
            temp.Attach += temp_attach;
            temp.Detach += temp_detach;
            temp.Error += temp_error;
            temp.TemperatureChange += temp_change;
            temp.Open();

            if (defaultSettings.emailServerIsSet)
            {
                emailServerText.Text = defaultSettings.emailServer;
            }
            if (defaultSettings.usernameIsSet)
            {
                usernameText.Text = defaultSettings.username;
            }
            else
            {
                usernameText.Text = "FROM@EMAIL.COM";
            }
            if (defaultSettings.passwordIsSet)
            {
                passwordText.Text = defaultSettings.password;
            }
            else
            {
                passwordText.Text = "";
            }
            if (defaultSettings.temperatureLimitIsSet)
            {
                temperatureLimitText.Text = defaultSettings.temperatureLimit.ToString();
            }
            if (defaultSettings.notifyEmailIsSet)
            {
                notifyEmailText.Text = defaultSettings.notifyEmail;
            }
            else
            {
                notifyEmailText.Text = "TO@EMAIL.COM";
            }

            updateStatus();
        }

        private void Form1_Resize(object sender, EventArgs e)
        {
            if (FormWindowState.Minimized == this.WindowState)
                Hide();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void notifyIcon1_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            if (WindowState == FormWindowState.Minimized)
            {
                Show();
                WindowState = FormWindowState.Normal;
            }
            else
            {
                WindowState = FormWindowState.Minimized;
                Hide();
            }
        }

        private void exitToolStripMenuItem_Click_1(object sender, EventArgs e)
        {
            Application.Exit();
        }
        #endregion

        #region TEMPERATURE_EVENTS
        void temp_change(object sender, Phidget22.Events.TemperatureSensorTemperatureChangeEventArgs e)
        {
            temperatureLabel.Text = e.Temperature.ToString("F1") + "°C";
            if (isRunning())
            {
                if (e.Temperature < defaultSettings.temperatureLimit)
                {
                    if (++count > 9 && timeOK)//count is reset in timer
                    { //if temperature is below limit for 5 minutes
                        Log.WriteLine(LogLevel.Info, "Send email.");
                        timeOK = false;
                        sendEmail(e.Temperature);
                        timer1.Enabled = true;
                    }
                }
                else
                    count = 0;
            }
        }

        void temp_attach(object sender, Phidget22.Events.AttachEventArgs e)
        {
            temp.DataInterval = 30000; //measure temperature every 30 seconds
        }

        void temp_detach(object sender, Phidget22.Events.DetachEventArgs e)
        {

        }

        void temp_error(object sender, Phidget22.Events.ErrorEventArgs e)
        {

        }
        #endregion

        #region EMAIL
        bool IsValidEmail(string email)
        {
            try
            {
                var addr = new System.Net.Mail.MailAddress(email);
                return addr.Address == email;
            }
            catch
            {
                return false;
            }
        }

        private void sendEmail(double temperature)
        {
            try
            {
                // Command line argument must the the SMTP host.
                SmtpClient client = new SmtpClient();
                client.Port = 587;
                client.Host = defaultSettings.emailServer;
                client.EnableSsl = true;
                client.Timeout = 10000;
                client.DeliveryMethod = SmtpDeliveryMethod.Network;
                client.UseDefaultCredentials = false;
                client.Credentials = new System.Net.NetworkCredential(defaultSettings.username, defaultSettings.password);

                MailMessage mm = new MailMessage(defaultSettings.username, defaultSettings.notifyEmail, "Temperature Warning", "Temperature has dropped below limit. Last recorded temperature was: "
                    + temperature + "°C");
                mm.BodyEncoding = UTF8Encoding.UTF8;
                mm.DeliveryNotificationOptions = DeliveryNotificationOptions.OnFailure;

                client.Send(mm);
            }
            catch (Exception ex)
            {

            }
        }
        #endregion

        #region TIMER
        private void timer1_Tick(object sender, EventArgs e)
        {
            count = 0; //reset count
            timer1.Enabled = false;
            timeOK = true;
            Log.WriteLine(LogLevel.Info, "Timer tick");
        }
        #endregion

        #region BUTTON_EVENTS
        private void testButton_Click(object sender, EventArgs e)
        {
            sendEmail(-273);
        }

        private void tabControl1_SelectedIndexChanged(object sender, EventArgs e)
        {
            updateStatus();
        }

        private void saveGeneralButton_Click(object sender, EventArgs e)
        {
            double temperatureLimit;
            if (double.TryParse(temperatureLimitText.Text, out temperatureLimit))
            {
                defaultSettings.temperatureLimit = temperatureLimit;
                defaultSettings.temperatureLimitIsSet = true;
                defaultSettings.Save();
            }
            else
            {
                //invalid
                MessageBox.Show("Please enter a valid temperature limit.");
                temperatureLimitText.Text = "";
                return;
            }

            if (IsValidEmail(notifyEmailText.Text))
            {
                defaultSettings.notifyEmail = notifyEmailText.Text;
                defaultSettings.notifyEmailIsSet = true;
                defaultSettings.Save();
            }
            else
            {
                //invalid
                MessageBox.Show("Please enter a valid email.");
                notifyEmailText.Text = "";
                usernameText.Text = "FROM@EMAIL.COM";
                return;
            }


            MessageBox.Show("Settings have been saved.");
            updateStatus();
        }

        private void saveEmailButton_Click(object sender, EventArgs e)
        {
            if (IsValidEmail(usernameText.Text))
            {
                defaultSettings.username = usernameText.Text;
                defaultSettings.usernameIsSet = true;
                defaultSettings.Save();
            }
            else
            {
                //invalid
                MessageBox.Show("Please enter a valid email.");
                usernameText.Text = "FROM@EMAIL.COM";
                return;
            }

            if (passwordText.Text != "")
            {
                defaultSettings.password = passwordText.Text;
                defaultSettings.passwordIsSet = true;
                defaultSettings.Save();
            }

            if (emailServerText.Text != "")
            {
                defaultSettings.emailServer = emailServerText.Text;
                defaultSettings.emailServerIsSet = true;
                defaultSettings.Save();
            }
            MessageBox.Show("Settings have been saved.");
            updateStatus();
        }
        #endregion

        private void updateStatus()
        {
            if (isRunning())
            {
                activeText.Text = "Running";
                activeText.ForeColor = Color.Green;
                notifyIcon1.Text = "Temperature Monitor Running: " + temp.Temperature.ToString("F1") + "°C";
                testButton.Visible = true;
            }
            else
            {
                activeText.Text = "NOT Running";
                activeText.ForeColor = Color.Red;
                notifyIcon1.Text = "NOT running";
                testButton.Visible = false;
            }
        }
        private bool isRunning()
        {
            if (defaultSettings.emailServerIsSet && defaultSettings.usernameIsSet && defaultSettings.passwordIsSet
                && defaultSettings.temperatureLimitIsSet && defaultSettings.notifyEmailIsSet
                && temp.Attached)
                return true;
            else
                return false;
        }
    }
}
